<?php

namespace Drp\NativeEventApp\Auth;

use Carbon\Carbon;
use Drp\NativeEventApp\Exceptions\InvalidAuthIdentifier;

class MagicLinkSigner implements \Drp\NativeEventApp\Contracts\MagicLinkSigner
{
    /** @var string */
    private $key;

    public function __construct(string $key)
    {
        $this->key = $key;
    }

    public function sign(string $authId, string $expiry): string
    {
        $data = json_encode([
            'authId' => $authId,
            'expiry' => $expiry,
        ]);

        return hash_hmac('sha256', $data, $this->key);
    }

    /**
     * {@inheritdoc}
     */
    public function verify(string $authIdentifier, string $expiry, string $signature): bool
    {
        $check = $this->sign($authIdentifier, $expiry);

        throw_unless($check === $signature, InvalidAuthIdentifier::hasInvalidSignature());

        throw_if(Carbon::now()->getTimestamp() > $expiry, InvalidAuthIdentifier::hasExpired());

        return true;
    }
}
