<?php

namespace Drp\NativeEventApp;

use Drp\NativeEventApp\Contracts\Obfuscatable;
use Drp\NativeEventApp\Exceptions\InvalidEmailAddress;
use Illuminate\Support\Str;

class EmailAddress implements Obfuscatable
{
    /** @var string */
    private $email;

    public function __construct(?string $email)
    {
        throw_unless(self::isValid($email), InvalidEmailAddress::invalidFormat($email));

        $this->email = $email;
    }

    public static function isValid(?string $email): bool
    {
        return (bool) filter_var($email, FILTER_VALIDATE_EMAIL);
    }

    public function obfuscated(): string
    {
        $domain = Str::after($this->email, '@');
        $addressCharacters = str_split(Str::before($this->email, '@'));

        $hiddenEmail = $this->anonymiseLocalPart($addressCharacters);

        return "$hiddenEmail@$domain";
    }

    public function __toString(): string
    {
        return $this->email;
    }

    private function anonymiseLocalPart(array $addressCharacters): string
    {
        $firstCharacter = array_shift($addressCharacters);

        if (count($addressCharacters) < 3) {
            return $firstCharacter . '*';
        }

        $lastCharacter = array_pop($addressCharacters);
        $hiddenCharacters = array_fill(0, count($addressCharacters), '*');

        return $firstCharacter . implode($hiddenCharacters) . $lastCharacter;
    }
}
