<?php

namespace Drp\NativeEventApp\Http\Controllers;

use Drp\NativeEventApp\Models\FileUpload;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\Storage;
use Mimey\MimeTypes;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\StreamedResponse;

class RetrieveFileUpload extends Controller
{
    public function __invoke(FileUpload $fileUpload): StreamedResponse
    {
        return new StreamedResponse(
            static function () use ($fileUpload): void {
                $stream = Storage::disk($fileUpload->disk)->readStream($fileUpload->path);
                $output = fopen('php://output', 'wb+');

                stream_copy_to_stream($stream, $output);
                fclose($stream);
                fclose($output);
            },
            Response::HTTP_OK,
            $this->getResponseHeaders($fileUpload)
        );
    }

    private function getResponseHeaders(FileUpload $fileUpload): array
    {
        $mimeType = Storage::disk($fileUpload->disk)->mimeType($fileUpload->path);

        return [
            'Content-Disposition' => $this->getContentDisposition($fileUpload->uuid, $mimeType),
            'Content-Length' => Storage::disk($fileUpload->disk)->size($fileUpload->path),
            'Content-Type' => $mimeType,
        ];
    }

    private function getContentDisposition(string $fileName, string $mimeType): string
    {
        if (request()->query('download', false) === false) {
            return 'inline';
        }

        $mimes = new MimeTypes();

        return 'attachment; filename="' . $fileName . $mimes->getExtension($mimeType) . '"';
    }
}
