<?php

namespace Drp\NativeEventApp\Modules\Types\Actions;

use Illuminate\Contracts\Support\Arrayable;

class Action implements Arrayable
{
    public const NAVIGATE_PUSH = 'NAVIGATE_PUSH';
    public const NAVIGATE_POP = 'NAVIGATE_POP';
    public const MODAL_OPEN = 'MODAL_OPEN';
    public const MODAL_CLOSE = 'MODAL_CLOSE';
    public const OPEN_URL = 'OPEN_URL';
    public const SUBMIT = 'SUBMIT';

    /** @var string */
    private $type;

    /** @var array */
    private $payload;

    /**
     * @param string|int $identifier
     */
    public static function navigate($identifier): self
    {
        return new self(self::NAVIGATE_PUSH, ['id' => $identifier]);
    }

    public static function navigatePop(): self
    {
        return new self(self::NAVIGATE_POP, []);
    }

    /**
     * @param string|int $identifier
     */
    public static function openModal($identifier): self
    {
        return new self(self::MODAL_OPEN, ['id' => $identifier]);
    }

    public static function closeModal(): self
    {
        return new self(self::MODAL_CLOSE, []);
    }

    public static function openUrl(string $url): self
    {
        return new self(self::OPEN_URL, ['url' => $url]);
    }

    public static function submit(): self
    {
        return new self(self::SUBMIT, []);
    }

    public static function fromArray(array $action): self
    {
        return new self($action['type'], $action['payload']);
    }

    public function __construct(string $type, array $payload)
    {
        $this->type = $type;
        $this->payload = $payload;
    }

    public function type(): string
    {
        return $this->type;
    }

    public function payload(): array
    {
        return $this->payload;
    }

    public function toArray(): array
    {
        return [
            'type' => $this->type,
            'payload' => $this->payload,
        ];
    }
}
