<?php

namespace Drp\NativeEventApp\Test\Api;

use Carbon\Carbon;
use Drp\NativeEventApp\Models\AgendaItem;
use Spatie\Snapshots\MatchesSnapshots;

class AgendaTest extends GraphQLTestCase
{
    use MatchesSnapshots;

    public function setUp(): void
    {
        parent::setUp();

        Carbon::setTestNow('1st January 2019');
    }

    /** @test */
    public function can_get_agendas(): void
    {
        $agendaItems = factory(AgendaItem::class, 2)->create();

        $query = '{
            events(id:1){
                id,
                agendas {
                    key
                    items {
                        title,
                        description,
                        startTime,
                        endTime
                    }
                }
            }
        }';

        $this->assertMatchesJsonSnapshot($this->authenticatedGraphQlRequest(
            $query,
            $agendaItems->first()->agenda->event
        ));
    }

    /** @test */
    public function allows_for_null_values(): void
    {
        $agendaItems = factory(AgendaItem::class, 2)->create([
            'description' => null,
            'startTime' => null,
            'endTime' => null,
        ]);

        $query = '{
            events(id:1){
                id,
                agendas {
                    key
                    items {
                        title,
                        description,
                        startTime,
                        endTime
                    }
                }
            }
        }';

        $this->assertMatchesJsonSnapshot($this->authenticatedGraphQlRequest(
            $query,
            $agendaItems->first()->agenda->event
        ));
    }
}
