<?php

namespace Drp\NativeEventApp\Test\Unit\Modules\Types\Actions;

use Drp\NativeEventApp\Modules\Types\Actions\Action;
use Drp\NativeEventApp\Test\TestCase;

class ActionTest extends TestCase
{
    /** @test */
    public function it_casts_to_an_array(): void
    {
        $action = new Action('TEST_TYPE', ['id' => 1]);

        $result = $action->toArray();

        $this->assertArrayResponse($result, 'TEST_TYPE', [ 'id' => 1 ]);
    }

    /** @test */
    public function it_can_get_the_payload(): void
    {
        $payload = ['id' => 1];
        $action = new Action('TEST_TYPE', $payload);

        $result = $action->payload();

        $this->assertEquals($payload, $result);
    }

    /** @test */
    public function it_can_get_the_type(): void
    {
        $type = 'TEST_TYPE';
        $action = new Action($type, []);

        $result = $action->type();

        $this->assertEquals($type, $result);
    }

    /** @test */
    public function it_can_create_an_action_from_an_array(): void
    {
        $action = Action::fromArray([
            'type' => 'TEST_TYPE',
            'payload' => [
                'id' => 1,
            ],
        ]);

        $result = $action->toArray();

        $this->assertArrayResponse($result, 'TEST_TYPE', [ 'id' => 1 ]);
    }

    /** @test */
    public function it_can_create_a_navigate_action(): void
    {
        $action = Action::navigate(1);

        $result = $action->toArray();

        $this->assertArrayResponse($result, Action::NAVIGATE_PUSH, [ 'id' => 1 ]);
    }

    /** @test */
    public function it_can_create_a_navigate_pop_action(): void
    {
        $action = Action::navigatePop();

        $result = $action->toArray();

        $this->assertArrayResponse($result, Action::NAVIGATE_POP, []);
    }

    /** @test */
    public function it_can_create_a_open_modal_action(): void
    {
        $action = Action::openModal(1);

        $result = $action->toArray();

        $this->assertArrayResponse($result, Action::MODAL_OPEN, [ 'id' => 1 ]);
    }

    /** @test */
    public function it_can_create_a_close_modal_action(): void
    {
        $action = Action::closeModal();

        $result = $action->toArray();

        $this->assertArrayResponse($result, Action::MODAL_CLOSE, []);
    }

    /** @test */
    public function it_can_create_a_open_url_action(): void
    {
        $action = Action::openUrl('https://www.google.co.uk');

        $result = $action->toArray();

        $this->assertArrayResponse($result, Action::OPEN_URL, [
            'url' => 'https://www.google.co.uk',
        ]);
    }

    /** @test */
    public function it_can_create_a_submit_action(): void
    {
        $action = Action::submit();

        $result = $action->toArray();

        $this->assertArrayResponse($result, Action::SUBMIT, []);
    }

    private function assertArrayResponse(array $result, string $type, array $payload): void
    {
        $this->assertArrayHasKey('type', $result);
        $this->assertArrayHasKey('payload', $result);
        $this->assertEquals($type, $result['type']);
        $this->assertEquals($payload, $result['payload']);
    }
}
