<?php

namespace Drp\NativeEventApp\Test\Unit\Modules\Types\Form;

use Drp\NativeEventApp\Modules\Modules;
use Drp\NativeEventApp\Modules\Types\Actions\Action;
use Drp\NativeEventApp\Modules\Types\Form\FileInput;
use Drp\NativeEventApp\Modules\Types\Form\Form;
use Drp\NativeEventApp\Modules\Types\Form\Select;
use Drp\NativeEventApp\Modules\Types\Form\TextInput;
use Drp\NativeEventApp\Modules\Types\Text;
use Drp\NativeEventApp\Test\TestCase;

class FormTest extends TestCase
{
    /** @test */
    public function form_can_be_serialized(): void
    {
        $form = Form::make('form_1')
            ->onSubmit(Action::navigate(1))
            ->children(Modules::make([
                TextInput::make('input')
                    ->label('Answer me this')
                    ->placeholder('Enter input')
                    ->value('Default value'),
                TextInput::make('required-input')
                    ->label('You must answer this')
                    ->placeholder('Enter input')
                    ->value('Default value')
                    ->required(),
                Text::make('My test'),
                Select::make('select')
                    ->label('Select dropdown')
                    ->options([
                        'Label One' => '1',
                        'Label Two' => '2',
                    ])
                    ->placeholder('Please select')
                    ->value('1'),
                FileInput::make('selfie')
                    ->placeholder('Tap to upload a photo'),
            ]));

        $this->assertEquals(json_encode([
            'type' => 'Form',
            'name' => 'form_1',
            'onSubmit' => [
                'type' => Action::NAVIGATE_PUSH,
                'payload' => ['id' => 1],
            ],
            'children' => [
                [
                    'type' => 'TextInput',
                    'label' => 'Answer me this',
                    'value' => 'Default value',
                    'name' => 'input',
                    'required' => false,
                    'styles' => [],
                    'className' => [],
                    'viewableBy' => [],
                    'placeholder' => 'Enter input',
                ],
                [
                    'type' => 'TextInput',
                    'label' => 'You must answer this',
                    'value' => 'Default value',
                    'name' => 'required-input',
                    'required' => true,
                    'styles' => [],
                    'className' => [],
                    'viewableBy' => [],
                    'placeholder' => 'Enter input',
                ],
                [
                    'type' => 'Text',
                    'text' => 'My test',
                    'styles' => [],
                    'className' => [],
                    'viewableBy' => [],
                ],
                [
                    'type' => 'Select',
                    'options' => [
                        ['label' => 'Label One', 'value' => '1'],
                        ['label' => 'Label Two', 'value' => '2'],
                    ],
                    'label' => 'Select dropdown',
                    'value' => '1',
                    'name' => 'select',
                    'required' => false,
                    'styles' => [],
                    'className' => [],
                    'viewableBy' => [],
                    'placeholder' => 'Please select',
                ],
                [
                    'type' => 'FileInput',
                    'label' => null,
                    'value' => null,
                    'name' => 'selfie',
                    'required' => false,
                    'styles' => [],
                    'className' => [],
                    'viewableBy' => [],
                    'placeholder' => 'Tap to upload a photo',
                ],
            ],
            'styles' => [],
            'className' => [],
            'viewableBy' => [],
        ]), $form->toJson());
    }
}
